@@ -33,7 +33,7 @@
 
 #include "kx.h"
 
-RCSID("$Id: kxd.c,v 1.76.2.1 2005/07/13 20:47:19 lha Exp $");
+RCSID("$Id: kxd.c 17450 2006-05-05 11:11:43Z lha $");
 
 static pid_t wait_on_pid = -1;
 static int   done        = 0;
@@ -119,7 +119,7 @@ recv_conn (int sock, kx_context *kc,
      int ret = 1;
      int flags;
      int len;
-     u_int32_t tmp32;
+     uint32_t tmp32;
 
      memset(kc, 0, sizeof(*kc));
      *nsockets = 0;
@@ -538,11 +538,11 @@ doit_passive (kx_context *kc,
 		    if (sockets[i].flags == TCP) {
 			struct sockaddr_storage __ss_peer;
 			struct sockaddr *peer = (struct sockaddr*)&__ss_peer;
-			socklen_t len = sizeof(__ss_peer);
+			socklen_t slen = sizeof(__ss_peer);
 
 			fd = accept (sockets[i].fd,
 				     peer,
-				     &len);
+				     &slen);
 			if (fd < 0 && errno != EINTR)
 			    syslog (LOG_ERR, "accept: %m");
 
@@ -725,13 +725,13 @@ int
 main (int argc, char **argv)
 {
     int port;
-    int optind = 0;
+    int optidx = 0;
 
     setprogname (argv[0]);
     roken_openlog ("kxd", LOG_ODELAY | LOG_PID, LOG_DAEMON);
 
     if (getarg (args, sizeof(args) / sizeof(args[0]), argc, argv,
-		&optind))
+		&optidx))
 	usage (1);
 
     if (help_flag)
