@@ -31,7 +31,7 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: rsh_locl.h,v 1.37.2.1 2005/12/28 18:00:05 lha Exp $ */
+/* $Id: rsh_locl.h 19033 2006-11-14 09:47:07Z lha $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -108,16 +108,15 @@ struct krb5_pk_cert;
 struct ContentInfo;
 struct _krb5_krb_auth_data;
 struct krb5_dh_moduli;
+struct krb5_plugin;
+enum plugin_type;
+#include "crypto-headers.h"
 #include <krb5-private.h> /* for _krb5_{get,put}_int */
 #endif
 #if defined(KRB4) || defined(KRB5)
 #include <kafs.h>
 #endif
 
-#ifndef _PATH_NOLOGIN
-#define _PATH_NOLOGIN   "/etc/nologin"
-#endif
-
 #ifndef _PATH_BSHELL
 #define _PATH_BSHELL	"/bin/sh"
 #endif
@@ -126,9 +125,7 @@ struct krb5_dh_moduli;
 #define _PATH_DEFPATH	"/usr/bin:/bin"
 #endif
 
-#ifndef _PATH_ETC_ENVIRONMENT
-#define _PATH_ETC_ENVIRONMENT SYSCONFDIR "/environment"
-#endif
+#include "loginpaths.h"
 
 /*
  *
