@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2006 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-2007 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -32,10 +32,17 @@
  */
 
 #include "rsh_locl.h"
-RCSID("$Id: rshd.c,v 1.60.2.2 2006/02/03 14:48:10 lha Exp $");
+#include "login_locl.h"
+RCSID("$Id: rshd.c 19653 2007-01-03 23:29:48Z lha $");
 
 int
 login_access( struct passwd *user, char *from);
+int
+read_limits_conf(const char *file, const struct passwd *pwd);
+
+#ifdef NEED_IRUSEROK_PROTO
+int iruserok(uint32_t, int, const char *, const char *);
+#endif
 
 enum auth_method auth_method;
 
@@ -310,7 +317,7 @@ recv_krb5_auth (int s, u_char *buf,
 		char **server_username,
 		char **cmd)
 {
-    u_int32_t len;
+    uint32_t len;
     krb5_auth_context auth_context = NULL;
     krb5_ticket *ticket;
     krb5_error_code status;
@@ -824,6 +831,12 @@ doit (void)
 	syslog(LOG_ERR, "setpcred() failure: %m");
 #endif /* HAVE_SETPCRED */
 
+    /* Apply limits if not root */
+    if(pwd->pw_uid != 0) {
+	 const char *file = _PATH_LIMITS_CONF;
+	 read_limits_conf(file, pwd);
+    }
+
     if (initgroups (pwd->pw_name, pwd->pw_gid) < 0)
 	fatal (s, "initgroups", "Login incorrect.");
 
