@@ -34,7 +34,7 @@
 #include "test_locl.h"
 #include <gssapi.h>
 #include "gss_common.h"
-RCSID("$Id: gssapi_client.c,v 1.19 2003/09/10 09:32:42 lha Exp $");
+RCSID("$Id: gssapi_client.c 16227 2005-10-25 10:55:21Z lha $");
 
 static int
 do_trans (int sock, gss_ctx_id_t context_hdl)
@@ -115,7 +115,9 @@ proto (int sock, const char *hostname, const char *service)
 
     name_token.length = asprintf ((char **)&name_token.value,
 				  "%s@%s", service, hostname);
-
+    if (name_token.length == -1)
+	errx(1, "malloc - out of memory");
+	
     maj_stat = gss_import_name (&min_stat,
 				&name_token,
 				GSS_C_NT_HOSTBASED_SERVICE,
