@@ -1,9 +1,10 @@
 #!/bin/sh
 #
-# $Id: install-catman.sh,v 1.3.14.1 2005/10/26 10:19:11 lha Exp $
+# $Id: install-catman.sh 19918 2007-01-15 14:59:23Z lha $
 #
 # install preformatted manual pages
 
+cmd="$1"; shift
 INSTALL_DATA="$1"; shift
 mkinstalldirs="$1"; shift
 srcdir="$1"; shift
@@ -19,14 +20,22 @@ for f in "$@"; do
 	c="$base.cat$section"
 
 	if test "$catinstall" = yes -a -f "$srcdir/$c"; then
-		if test \! -d "$catdir"; then
-			eval "$mkinstalldirs $catdir"
+		if test "$cmd" = install ; then
+			if test \! -d "$catdir"; then
+				eval "$mkinstalldirs $catdir"
+			fi
+			eval "echo $INSTALL_DATA $srcdir/$c $catdir/$base.$suffix"
+			eval "$INSTALL_DATA $srcdir/$c $catdir/$base.$suffix"
+		elif test "$cmd" = uninstall ; then
+			eval "echo rm -f $catdir/$base.$suffix"
+			eval "rm -f $catdir/$base.$suffix"
 		fi
-		eval "echo $INSTALL_DATA $srcdir/$c $catdir/$base.$suffix"
-		eval "$INSTALL_DATA $srcdir/$c $catdir/$base.$suffix"
 	fi
 	for link in `sed -n -e '/SYNOPSIS/q;/DESCRIPTION/q;s/^\.Nm \([^ ]*\).*/\1/p' $srcdir/$f`; do
-		if [ "$link" != "$base" ]; then
+		if test "$link" = "$base" ; then
+			continue
+		fi
+		if test "$cmd" = install ; then
 			target="$mandir/$link.$section"
 			for cmd in "ln -f $mandir/$base.$section $target" \
 				   "ln -s $base.$section $target" \
@@ -49,6 +58,15 @@ for f in "$@"; do
 					fi
 				done
 			fi
+		elif test "$cmd" = uninstall ; then
+			target="$mandir/$link.$section"
+			eval "echo rm -f $target"
+			eval "rm -f $target"
+			if test "$catinstall" = yes; then
+				target="$catdir/$link.$suffix"
+				eval "echo rm -f $target"
+				eval "rm -f $target"
+			fi
 		fi
 	done
 done
