@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 - 2005 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997 - 2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -35,11 +35,11 @@
 
 #include <krb5-v4compat.h>
 
-RCSID("$Id: kerberos4.c,v 1.52 2005/04/23 20:11:55 lha Exp $");
+RCSID("$Id: kerberos4.c 18349 2006-10-08 13:43:52Z lha $");
 
 #ifndef swap32
-static u_int32_t
-swap32(u_int32_t x)
+static uint32_t
+swap32(uint32_t x)
 {
     return ((x << 24) & 0xff000000) |
 	((x << 8) & 0xff0000) |
@@ -49,7 +49,7 @@ swap32(u_int32_t x)
 #endif /* swap32 */
 
 int
-maybe_version4(unsigned char *buf, int len)
+_kdc_maybe_version4(unsigned char *buf, int len)
 {
     return len > 0 && *buf == 4;
 }
@@ -62,41 +62,56 @@ make_err_reply(krb5_context context, krb5_data *reply,
 			   kdc_time, code, msg, reply);
 }
 
+struct valid_princ_ctx {
+    krb5_kdc_configuration *config;
+    unsigned flags;
+};
+
 static krb5_boolean
-valid_princ(krb5_context context, krb5_principal princ)
+valid_princ(krb5_context context,
+	    void *funcctx,
+	    krb5_principal princ)
 {
+    struct valid_princ_ctx *ctx = funcctx;
     krb5_error_code ret;
     char *s;
-    hdb_entry *ent;
+    hdb_entry_ex *ent;
 
     ret = krb5_unparse_name(context, princ, &s);
     if (ret)
 	return FALSE;
-    ret = db_fetch(princ, &ent);
+    ret = _kdc_db_fetch(context, ctx->config, princ, ctx->flags, NULL, &ent);
     if (ret) {
-	kdc_log(7, "Lookup %s failed: %s", s,
+	kdc_log(context, ctx->config, 7, "Lookup %s failed: %s", s,
 		krb5_get_err_text (context, ret));
 	free(s);
 	return FALSE;
     }
-    kdc_log(7, "Lookup %s succeeded", s);
+    kdc_log(context, ctx->config, 7, "Lookup %s succeeded", s);
     free(s);
-    free_ent(ent);
+    _kdc_free_ent(context, ent);
     return TRUE;
 }
 
 krb5_error_code
-db_fetch4(const char *name, const char *instance, const char *realm,
-	  hdb_entry **ent)
+_kdc_db_fetch4(krb5_context context,
+	       krb5_kdc_configuration *config,
+	       const char *name, const char *instance, const char *realm,
+	       unsigned flags,
+	       hdb_entry_ex **ent)
 {
     krb5_principal p;
     krb5_error_code ret;
+    struct valid_princ_ctx ctx;
+
+    ctx.config = config;
+    ctx.flags = flags;
     
-    ret = krb5_425_conv_principal_ext(context, name, instance, realm, 
-				      valid_princ, 0, &p);
+    ret = krb5_425_conv_principal_ext2(context, name, instance, realm, 
+				       valid_princ, &ctx, 0, &p);
     if(ret)
 	return ret;
-    ret = db_fetch(p, ent);
+    ret = _kdc_db_fetch(context, config, p, flags, NULL, ent);
     krb5_free_principal(context, p);
     return ret;
 }
@@ -110,15 +125,17 @@ db_fetch4(const char *name, const char *instance, const char *realm,
  */
 
 krb5_error_code
-do_version4(unsigned char *buf,
-	    size_t len,
-	    krb5_data *reply,
-	    const char *from,
-	    struct sockaddr_in *addr)
+_kdc_do_version4(krb5_context context, 
+		 krb5_kdc_configuration *config,
+		 unsigned char *buf,
+		 size_t len,
+		 krb5_data *reply,
+		 const char *from,
+		 struct sockaddr_in *addr)
 {
     krb5_storage *sp;
     krb5_error_code ret;
-    hdb_entry *client = NULL, *server = NULL;
+    hdb_entry_ex *client = NULL, *server = NULL;
     Key *ckey, *skey;
     int8_t pvno;
     int8_t msg_type;
@@ -127,12 +144,13 @@ do_version4(unsigned char *buf,
     char *sname = NULL, *sinst = NULL;
     int32_t req_time;
     time_t max_life;
-    u_int8_t life;
+    uint8_t life;
     char client_name[256];
     char server_name[256];
 
-    if(!enable_v4) {
-	kdc_log(0, "Rejected version 4 request from %s", from);
+    if(!config->enable_v4) {
+	kdc_log(context, config, 0,
+		"Rejected version 4 request from %s", from);
 	make_err_reply(context, reply, KDC_GEN_ERR, "function not enabled");
 	return 0;
     }
@@ -140,7 +158,8 @@ do_version4(unsigned char *buf,
     sp = krb5_storage_from_mem(buf, len);
     RCHECK(krb5_ret_int8(sp, &pvno), out);
     if(pvno != 4){
-	kdc_log(0, "Protocol version mismatch (krb4) (%d)", pvno);
+	kdc_log(context, config, 0,
+		"Protocol version mismatch (krb4) (%d)", pvno);
 	make_err_reply(context, reply, KDC_PKT_VER, "protocol mismatch");
 	goto out;
     }
@@ -161,37 +180,40 @@ do_version4(unsigned char *buf,
 	RCHECK(krb5_ret_int32(sp, &req_time), out1);
 	if(lsb)
 	    req_time = swap32(req_time);
-	RCHECK(krb5_ret_int8(sp, &life), out1);
+	RCHECK(krb5_ret_uint8(sp, &life), out1);
 	RCHECK(krb5_ret_stringz(sp, &sname), out1);
 	RCHECK(krb5_ret_stringz(sp, &sinst), out1);
 	snprintf (client_name, sizeof(client_name),
 		  "%s.%s@%s", name, inst, realm);
 	snprintf (server_name, sizeof(server_name),
-		  "%s.%s@%s", sname, sinst, v4_realm);
+		  "%s.%s@%s", sname, sinst, config->v4_realm);
 	
-	kdc_log(0, "AS-REQ (krb4) %s from %s for %s",
+	kdc_log(context, config, 0, "AS-REQ (krb4) %s from %s for %s",
 		client_name, from, server_name);
 
-	ret = db_fetch4(name, inst, realm, &client);
+	ret = _kdc_db_fetch4(context, config, name, inst, realm, 
+			     HDB_F_GET_CLIENT, &client);
 	if(ret) {
-	    kdc_log(0, "Client not found in database: %s: %s",
+	    kdc_log(context, config, 0, "Client not found in database: %s: %s",
 		    client_name, krb5_get_err_text(context, ret));
 	    make_err_reply(context, reply, KERB_ERR_PRINCIPAL_UNKNOWN,
 			   "principal unknown");
 	    goto out1;
 	}
-	ret = db_fetch4(sname, sinst, v4_realm, &server);
+	ret = _kdc_db_fetch4(context, config, sname, sinst, config->v4_realm,
+			     HDB_F_GET_SERVER, &server);
 	if(ret){
-	    kdc_log(0, "Server not found in database: %s: %s",
+	    kdc_log(context, config, 0, "Server not found in database: %s: %s",
 		    server_name, krb5_get_err_text(context, ret));
 	    make_err_reply(context, reply, KERB_ERR_PRINCIPAL_UNKNOWN,
 			   "principal unknown");
 	    goto out1;
 	}
 
-	ret = check_flags (client, client_name,
-			   server, server_name,
-			   TRUE);
+	ret = _kdc_check_flags (context, config, 
+				client, client_name,
+				server, server_name,
+				TRUE);
 	if (ret) {
 	    /* good error code? */
 	    make_err_reply(context, reply, KERB_ERR_NAME_EXP,
@@ -199,15 +221,26 @@ do_version4(unsigned char *buf,
 	    goto out1;
 	}
 
+	if (config->enable_v4_per_principal &&
+	    client->entry.flags.allow_kerberos4 == 0)
+	{
+	    kdc_log(context, config, 0,
+		    "Per principal Kerberos 4 flag not turned on for %s",
+		    client_name);
+	    make_err_reply(context, reply, KERB_ERR_NULL_KEY,
+			   "allow kerberos4 flag required");
+	    goto out1;
+	}
+
 	/*
 	 * There's no way to do pre-authentication in v4 and thus no
 	 * good error code to return if preauthentication is required.
 	 */
 
-	if (require_preauth
-	    || client->flags.require_preauth
-	    || server->flags.require_preauth) {
-	    kdc_log(0,
+	if (config->require_preauth
+	    || client->entry.flags.require_preauth
+	    || server->entry.flags.require_preauth) {
+	    kdc_log(context, config, 0,
 		    "Pre-authentication required for v4-request: "
 		    "%s for %s",
 		    client_name, server_name);
@@ -216,9 +249,9 @@ do_version4(unsigned char *buf,
 	    goto out1;
 	}
 
-	ret = get_des_key(client, FALSE, FALSE, &ckey);
+	ret = _kdc_get_des_key(context, client, FALSE, FALSE, &ckey);
 	if(ret){
-	    kdc_log(0, "no suitable DES key for client");
+	    kdc_log(context, config, 0, "no suitable DES key for client");
 	    make_err_reply(context, reply, KDC_NULL_KEY, 
 			   "no suitable DES key for client");
 	    goto out1;
@@ -228,9 +261,9 @@ do_version4(unsigned char *buf,
 	/* this is not necessary with the new code in libkrb */
 	/* find a properly salted key */
 	while(ckey->salt == NULL || ckey->salt->salt.length != 0)
-	    ret = hdb_next_keytype2key(context, client, KEYTYPE_DES, &ckey);
+	    ret = hdb_next_keytype2key(context, &client->entry, KEYTYPE_DES, &ckey);
 	if(ret){
-	    kdc_log(0, "No version-4 salted key in database -- %s.%s@%s", 
+	    kdc_log(context, config, 0, "No version-4 salted key in database -- %s.%s@%s", 
 		    name, inst, realm);
 	    make_err_reply(context, reply, KDC_NULL_KEY, 
 			   "No version-4 salted key in database");
@@ -238,9 +271,9 @@ do_version4(unsigned char *buf,
 	}
 #endif
 	
-	ret = get_des_key(server, TRUE, FALSE, &skey);
+	ret = _kdc_get_des_key(context, server, TRUE, FALSE, &skey);
 	if(ret){
-	    kdc_log(0, "no suitable DES key for server");
+	    kdc_log(context, config, 0, "no suitable DES key for server");
 	    /* XXX */
 	    make_err_reply(context, reply, KDC_NULL_KEY, 
 			   "no suitable DES key for server");
@@ -248,10 +281,10 @@ do_version4(unsigned char *buf,
 	}
 
 	max_life = _krb5_krb_life_to_time(0, life);
-	if(client->max_life)
-	    max_life = min(max_life, *client->max_life);
-	if(server->max_life)
-	    max_life = min(max_life, *server->max_life);
+	if(client->entry.max_life)
+	    max_life = min(max_life, *client->entry.max_life);
+	if(server->entry.max_life)
+	    max_life = min(max_life, *server->entry.max_life);
 
 	life = krb_time_to_life(kdc_time, kdc_time + max_life);
     
@@ -268,7 +301,7 @@ do_version4(unsigned char *buf,
 				      0,
 				      name,
 				      inst,
-				      v4_realm,
+				      config->v4_realm,
 				      addr->sin_addr.s_addr,
 				      &session,
 				      life,
@@ -288,9 +321,9 @@ do_version4(unsigned char *buf,
 				    &session,
 				    sname,
 				    sinst,
-				    v4_realm,
+				    config->v4_realm,
 				    life,
-				    server->kvno % 255,
+				    server->entry.kvno % 255,
 				    &ticket,
 				    kdc_time,
 				    &ckey->key,
@@ -309,8 +342,8 @@ do_version4(unsigned char *buf,
 					  realm,
 					  req_time,
 					  0,
-					  client->pw_end ? *client->pw_end : 0,
-					  client->kvno % 256,
+					  client->entry.pw_end ? *client->entry.pw_end : 0,
+					  client->entry.kvno % 256,
 					  &cipher,
 					  reply);
 	krb5_data_free(&cipher);
@@ -327,7 +360,7 @@ do_version4(unsigned char *buf,
 	int32_t address;
 	size_t pos;
 	krb5_principal tgt_princ = NULL;
-	hdb_entry *tgt = NULL;
+	hdb_entry_ex *tgt = NULL;
 	Key *tkey;
 	time_t max_end, actual_end, issue_time;
 	
@@ -337,39 +370,45 @@ do_version4(unsigned char *buf,
 	RCHECK(krb5_ret_int8(sp, &kvno), out2);
 	RCHECK(krb5_ret_stringz(sp, &realm), out2);
 	
-	ret = krb5_425_conv_principal(context, "krbtgt", realm, v4_realm,
+	ret = krb5_425_conv_principal(context, "krbtgt", realm,
+				      config->v4_realm,
 				      &tgt_princ);
 	if(ret){
-	    kdc_log(0, "Converting krbtgt principal (krb4): %s", 
+	    kdc_log(context, config, 0,
+		    "Converting krbtgt principal (krb4): %s", 
 		    krb5_get_err_text(context, ret));
 	    make_err_reply(context, reply, KFAILURE, 
 			   "Failed to convert v4 principal (krbtgt)");
 	    goto out2;
 	}
 
-	ret = db_fetch(tgt_princ, &tgt);
+	ret = _kdc_db_fetch(context, config, tgt_princ,
+			    HDB_F_GET_KRBTGT, NULL, &tgt);
 	if(ret){
 	    char *s;
-	    s = kdc_log_msg(0, "Ticket-granting ticket not "
+	    s = kdc_log_msg(context, config, 0, "Ticket-granting ticket not "
 			    "found in database (krb4): krbtgt.%s@%s: %s", 
-			    realm, v4_realm,
+			    realm, config->v4_realm,
 			    krb5_get_err_text(context, ret));
 	    make_err_reply(context, reply, KFAILURE, s);
 	    free(s);
 	    goto out2;
 	}
 	
-	if(tgt->kvno % 256 != kvno){
-	    kdc_log(0, "tgs-req (krb4) with old kvno %d (current %d) for "
-		    "krbtgt.%s@%s", kvno, tgt->kvno % 256, realm, v4_realm);
+	if(tgt->entry.kvno % 256 != kvno){
+	    kdc_log(context, config, 0,
+		    "tgs-req (krb4) with old kvno %d (current %d) for "
+		    "krbtgt.%s@%s", kvno, tgt->entry.kvno % 256, 
+		    realm, config->v4_realm);
 	    make_err_reply(context, reply, KDC_AUTH_EXP,
 			   "old krbtgt kvno used");
 	    goto out2;
 	}
 
-	ret = get_des_key(tgt, TRUE, FALSE, &tkey);
+	ret = _kdc_get_des_key(context, tgt, TRUE, FALSE, &tkey);
 	if(ret){
-	    kdc_log(0, "no suitable DES key for krbtgt (krb4)");
+	    kdc_log(context, config, 0, 
+		    "no suitable DES key for krbtgt (krb4)");
 	    /* XXX */
 	    make_err_reply(context, reply, KDC_NULL_KEY, 
 			   "no suitable DES key for krbtgt");
@@ -384,15 +423,16 @@ do_version4(unsigned char *buf,
 	auth.data = buf;
 	auth.length = pos;
 
-	if (check_ticket_addresses)
+	if (config->check_ticket_addresses)
 	    address = addr->sin_addr.s_addr;
 	else
 	    address = 0;
 
-	ret = _krb5_krb_rd_req(context, &auth, "krbtgt", realm, v4_realm,
+	ret = _krb5_krb_rd_req(context, &auth, "krbtgt", realm, 
+			       config->v4_realm,
 			       address, &tkey->key, &ad);
 	if(ret){
-	    kdc_log(0, "krb_rd_req: %d", ret);
+	    kdc_log(context, config, 0, "krb_rd_req: %d", ret);
 	    make_err_reply(context, reply, ret, "failed to parse request");
 	    goto out2;
 	}
@@ -400,71 +440,81 @@ do_version4(unsigned char *buf,
 	RCHECK(krb5_ret_int32(sp, &req_time), out2);
 	if(lsb)
 	    req_time = swap32(req_time);
-	RCHECK(krb5_ret_int8(sp, &life), out2);
+	RCHECK(krb5_ret_uint8(sp, &life), out2);
 	RCHECK(krb5_ret_stringz(sp, &sname), out2);
 	RCHECK(krb5_ret_stringz(sp, &sinst), out2);
 	snprintf (server_name, sizeof(server_name),
 		  "%s.%s@%s",
-		  sname, sinst, v4_realm);
+		  sname, sinst, config->v4_realm);
 	snprintf (client_name, sizeof(client_name),
 		  "%s.%s@%s",
 		  ad.pname, ad.pinst, ad.prealm);
 
-	kdc_log(0, "TGS-REQ (krb4) %s from %s for %s",
+	kdc_log(context, config, 0, "TGS-REQ (krb4) %s from %s for %s",
 		client_name, from, server_name);
 	
 	if(strcmp(ad.prealm, realm)){
-	    kdc_log(0, "Can't hop realms (krb4) %s -> %s", realm, ad.prealm);
+	    kdc_log(context, config, 0, 
+		    "Can't hop realms (krb4) %s -> %s", realm, ad.prealm);
 	    make_err_reply(context, reply, KERB_ERR_PRINCIPAL_UNKNOWN, 
 			   "Can't hop realms");
 	    goto out2;
 	}
 
-	if (!enable_v4_cross_realm && strcmp(realm, v4_realm) != 0) {
-	    kdc_log(0, "krb4 Cross-realm %s -> %s disabled", realm, v4_realm);
+	if (!config->enable_v4_cross_realm && strcmp(realm, config->v4_realm) != 0) {
+	    kdc_log(context, config, 0, 
+		    "krb4 Cross-realm %s -> %s disabled",
+		    realm, config->v4_realm);
 	    make_err_reply(context, reply, KERB_ERR_PRINCIPAL_UNKNOWN, 
 			   "Can't hop realms");
 	    goto out2;
 	}
 
 	if(strcmp(sname, "changepw") == 0){
-	    kdc_log(0, "Bad request for changepw ticket (krb4)");
+	    kdc_log(context, config, 0, 
+		    "Bad request for changepw ticket (krb4)");
 	    make_err_reply(context, reply, KERB_ERR_PRINCIPAL_UNKNOWN, 
 			   "Can't authorize password change based on TGT");
 	    goto out2;
 	}
 	
-	ret = db_fetch4(ad.pname, ad.pinst, ad.prealm, &client);
+	ret = _kdc_db_fetch4(context, config, ad.pname, ad.pinst, ad.prealm,
+			     HDB_F_GET_CLIENT, &client);
 	if(ret && ret != HDB_ERR_NOENTRY) {
 	    char *s;
-	    s = kdc_log_msg(0, "Client not found in database: (krb4) %s: %s",
+	    s = kdc_log_msg(context, config, 0,
+			    "Client not found in database: (krb4) %s: %s",
 			    client_name, krb5_get_err_text(context, ret));
 	    make_err_reply(context, reply, KERB_ERR_PRINCIPAL_UNKNOWN, s);
 	    free(s);
 	    goto out2;
 	}
-	if (client == NULL && strcmp(ad.prealm, v4_realm) == 0) {
+	if (client == NULL && strcmp(ad.prealm, config->v4_realm) == 0) {
 	    char *s;
-	    s = kdc_log_msg(0, "Local client not found in database: (krb4) "
+	    s = kdc_log_msg(context, config, 0,
+			    "Local client not found in database: (krb4) "
 			    "%s", client_name);
 	    make_err_reply(context, reply, KERB_ERR_PRINCIPAL_UNKNOWN, s);
 	    free(s);
 	    goto out2;
 	}
 
-	ret = db_fetch4(sname, sinst, v4_realm, &server);
+	ret = _kdc_db_fetch4(context, config, sname, sinst, config->v4_realm,
+			     HDB_F_GET_SERVER, &server);
 	if(ret){
 	    char *s;
-	    s = kdc_log_msg(0, "Server not found in database (krb4): %s: %s",
+	    s = kdc_log_msg(context, config, 0,
+			    "Server not found in database (krb4): %s: %s",
 			    server_name, krb5_get_err_text(context, ret));
 	    make_err_reply(context, reply, KERB_ERR_PRINCIPAL_UNKNOWN, s);
 	    free(s);
 	    goto out2;
 	}
 
-	ret = check_flags (client, client_name,
-			   server, server_name,
-			   FALSE);
+	ret = _kdc_check_flags (context, config, 
+				client, client_name,
+				server, server_name,
+				FALSE);
 	if (ret) {
 	    /* good error code? */
 	    make_err_reply(context, reply, KERB_ERR_NAME_EXP,
@@ -472,9 +522,10 @@ do_version4(unsigned char *buf,
 	    goto out2;
 	}
 
-	ret = get_des_key(server, TRUE, FALSE, &skey);
+	ret = _kdc_get_des_key(context, server, TRUE, FALSE, &skey);
 	if(ret){
-	    kdc_log(0, "no suitable DES key for server (krb4)");
+	    kdc_log(context, config, 0, 
+		    "no suitable DES key for server (krb4)");
 	    /* XXX */
 	    make_err_reply(context, reply, KDC_NULL_KEY, 
 			   "no suitable DES key for server");
@@ -483,10 +534,10 @@ do_version4(unsigned char *buf,
 
 	max_end = _krb5_krb_life_to_time(ad.time_sec, ad.life);
 	max_end = min(max_end, _krb5_krb_life_to_time(kdc_time, life));
-	if(server->max_life)
-	    max_end = min(max_end, kdc_time + *server->max_life);
-	if(client && client->max_life)
-	    max_end = min(max_end, kdc_time + *client->max_life);
+	if(server->entry.max_life)
+	    max_end = min(max_end, kdc_time + *server->entry.max_life);
+	if(client && client->entry.max_life)
+	    max_end = min(max_end, kdc_time + *client->entry.max_life);
 	life = min(life, krb_time_to_life(kdc_time, max_end));
 	
 	issue_time = kdc_time;
@@ -541,9 +592,9 @@ do_version4(unsigned char *buf,
 					&session,
 					sname,
 					sinst,
-					v4_realm,
+					config->v4_realm,
 					life,
-					server->kvno % 255,
+					server->entry.kvno % 255,
 					&ticket,
 					issue_time,
 					&ad.session,
@@ -572,13 +623,13 @@ do_version4(unsigned char *buf,
 	if(tgt_princ)
 	    krb5_free_principal(context, tgt_princ);
 	if(tgt)
-	    free_ent(tgt);
+	    _kdc_free_ent(context, tgt);
 	break;
     }
     case AUTH_MSG_ERR_REPLY:
 	break;
     default:
-	kdc_log(0, "Unknown message type (krb4): %d from %s", 
+	kdc_log(context, config, 0, "Unknown message type (krb4): %d from %s", 
 		msg_type, from);
 	
 	make_err_reply(context, reply, KFAILURE, "Unknown message type");
@@ -595,16 +646,18 @@ do_version4(unsigned char *buf,
     if(sinst)
 	free(sinst);
     if(client)
-	free_ent(client);
+	_kdc_free_ent(context, client);
     if(server)
-	free_ent(server);
+	_kdc_free_ent(context, server);
     krb5_storage_free(sp);
     return 0;
 }
 
 krb5_error_code
-encode_v4_ticket(void *buf, size_t len, const EncTicketPart *et,
-		 const PrincipalName *service, size_t *size)
+_kdc_encode_v4_ticket(krb5_context context, 
+		      krb5_kdc_configuration *config,
+		      void *buf, size_t len, const EncTicketPart *et,
+		      const PrincipalName *service, size_t *size)
 {
     krb5_storage *sp;
     krb5_error_code ret;
@@ -613,7 +666,8 @@ encode_v4_ticket(void *buf, size_t len, const EncTicketPart *et,
 
     {
 	krb5_principal princ;
-	_krb5_principalname2krb5_principal(&princ,
+	_krb5_principalname2krb5_principal(context,
+					   &princ,
 					   *service,
 					   et->crealm);
 	ret = krb5_524_conv_principal(context, 
@@ -625,7 +679,8 @@ encode_v4_ticket(void *buf, size_t len, const EncTicketPart *et,
 	if(ret)
 	    return ret;
 
-	_krb5_principalname2krb5_principal(&princ,
+	_krb5_principalname2krb5_principal(context,
+					   &princ,
 					   et->cname,
 					   et->crealm);
 				     
@@ -690,8 +745,9 @@ encode_v4_ticket(void *buf, size_t len, const EncTicketPart *et,
 }
 
 krb5_error_code
-get_des_key(hdb_entry *principal, krb5_boolean is_server, 
-	    krb5_boolean prefer_afs_key, Key **ret_key)
+_kdc_get_des_key(krb5_context context, 
+		 hdb_entry_ex *principal, krb5_boolean is_server, 
+		 krb5_boolean prefer_afs_key, Key **ret_key)
 {
     Key *v5_key = NULL, *v4_key = NULL, *afs_key = NULL, *server_key = NULL;
     int i;
@@ -705,7 +761,7 @@ get_des_key(hdb_entry *principal, krb5_boolean is_server,
 		afs_key == NULL || server_key == NULL);
 	++i) {
 	Key *key = NULL;
-	while(hdb_next_enctype2key(context, principal, etypes[i], &key) == 0) {
+	while(hdb_next_enctype2key(context, &principal->entry, etypes[i], &key) == 0) {
 	    if(key->salt == NULL) {
 		if(v5_key == NULL)
 		    v5_key = key;
