@@ -32,7 +32,7 @@
  */
 
 #include "kpasswd_locl.h"
-RCSID("$Id: kpasswdd.c,v 1.62 2005/04/22 11:03:11 lha Exp $");
+RCSID("$Id: kpasswdd.c 17571 2006-05-13 18:51:41Z lha $");
 
 #include <kadm5/admin.h>
 #ifdef HAVE_SYS_UN_H
@@ -74,7 +74,7 @@ send_reply (int s,
 {
     struct msghdr msghdr;
     struct iovec iov[3];
-    u_int16_t len, ap_rep_len;
+    uint16_t len, ap_rep_len;
     u_char header[6];
     u_char *p;
 
@@ -120,7 +120,7 @@ send_reply (int s,
 
 static int
 make_result (krb5_data *data,
-	     u_int16_t result_code,
+	     uint16_t result_code,
 	     const char *expl)
 {
     krb5_data_zero (data);
@@ -144,7 +144,7 @@ reply_error (krb5_realm realm,
 	     struct sockaddr *sa,
 	     int sa_size,
 	     krb5_error_code error_code,
-	     u_int16_t result_code,
+	     uint16_t result_code,
 	     const char *expl)
 {
     krb5_error_code ret;
@@ -189,7 +189,7 @@ reply_priv (krb5_auth_context auth_context,
 	    int s,
 	    struct sockaddr *sa,
 	    int sa_size,
-	    u_int16_t result_code,
+	    uint16_t result_code,
 	    const char *expl)
 {
     krb5_error_code ret;
@@ -231,7 +231,7 @@ reply_priv (krb5_auth_context auth_context,
 static void
 change (krb5_auth_context auth_context,
 	krb5_principal admin_principal,
-	u_int16_t version,
+	uint16_t version,
 	int s,
 	struct sockaddr *sa,
 	int sa_size,
@@ -334,6 +334,9 @@ change (krb5_auth_context auth_context,
 	goto out;
     }
 
+    conf.realm = principal->realm;
+    conf.mask |= KADM5_CONFIG_REALM;
+
     ret = kadm5_init_with_password_ctx(context, 
 				       admin,
 				       NULL,
@@ -400,9 +403,11 @@ change (krb5_auth_context auth_context,
     krb5_free_data (context, pwd_data);
     pwd_data = NULL;
     if (ret) {
-	krb5_warn (context, ret, "kadm5_s_chpass_principal_cond");
-	reply_priv (auth_context, s, sa, sa_size, KRB5_KPASSWD_HARDERROR,
-		    "Internal error");
+	char *str = krb5_get_error_message(context, ret);
+	krb5_warnx(context, "kadm5_s_chpass_principal_cond: %s", str);
+	reply_priv (auth_context, s, sa, sa_size, KRB5_KPASSWD_SOFTERROR,
+		    str ? str : "Internal error");
+	krb5_free_error_string(context, str);
 	goto out;
     }
     reply_priv (auth_context, s, sa, sa_size, KRB5_KPASSWD_SUCCESS,
@@ -425,7 +430,7 @@ verify (krb5_auth_context *auth_context,
 	krb5_keytab keytab,
 	krb5_ticket **ticket,
 	krb5_data *out_data,
-	u_int16_t *version,
+	uint16_t *version,
 	int s,
 	struct sockaddr *sa,
 	int sa_size,
@@ -433,7 +438,7 @@ verify (krb5_auth_context *auth_context,
 	size_t len)
 {
     krb5_error_code ret;
-    u_int16_t pkt_len, pkt_ver, ap_req_len;
+    uint16_t pkt_len, pkt_ver, ap_req_len;
     krb5_data ap_req_data;
     krb5_data krb_priv_data;
     krb5_realm *r;
@@ -551,7 +556,7 @@ process (krb5_realm *realms,
     krb5_data out_data;
     krb5_ticket *ticket;
     krb5_address other_addr;
-    u_int16_t version;
+    uint16_t version;
 
 
     krb5_data_zero (&out_data);
@@ -630,7 +635,7 @@ doit (krb5_keytab keytab, int port)
     maxfd = -1;
     FD_ZERO(&real_fdset);
     for (i = 0; i < n; ++i) {
-	int sa_size = sizeof(__ss);
+	krb5_socklen_t sa_size = sizeof(__ss);
 
 	krb5_addr2sockaddr (context, &addrs.val[i], sa, &sa_size, port);
 	
@@ -687,6 +692,11 @@ doit (krb5_keytab keytab, int port)
 			 buf, ret);
 	    }
     }
+
+    for (i = 0; i < n; ++i)
+	close(sockets[i]);
+    free(sockets);
+
     krb5_free_addresses (context, &addrs);
     krb5_free_host_realm (context, realms);
     krb5_free_context (context);
