@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2004 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-2006 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -32,7 +32,7 @@
  */
 
 #include "kuser_locl.h"
-RCSID("$Id: kinit.c,v 1.121 2005/06/14 00:14:43 lha Exp $");
+RCSID("$Id: kinit.c 19323 2006-12-12 16:35:41Z lha $");
 
 #ifndef KRB4
 #include "krb5-v4compat.h"
@@ -42,6 +42,9 @@ struct krb5_pk_identity;
 struct krb5_pk_cert;
 struct ContentInfo;
 struct _krb5_krb_auth_data;
+struct krb5_dh_moduli;
+struct krb5_plugin;
+enum plugin_type;
 #include "krb5-private.h"
 
 int forwardable_flag	= -1;
@@ -52,12 +55,12 @@ int pac_flag		= -1;
 int validate_flag	= 0;
 int version_flag	= 0;
 int help_flag		= 0;
-int addrs_flag		= 1;
+int addrs_flag		= -1;
 struct getarg_strings extra_addresses;
 int anonymous_flag	= 0;
 char *lifetime 		= NULL;
 char *renew_life	= NULL;
-char *server		= NULL;
+char *server_str	= NULL;
 char *cred_cache	= NULL;
 char *start_str		= NULL;
 struct getarg_strings etype_str;
@@ -67,9 +70,10 @@ int do_afslog		= -1;
 int get_v4_tgt		= -1;
 int convert_524		= 0;
 int fcache_version;
+char *password_file	= NULL;
 char *pk_user_id	= NULL;
 char *pk_x509_anchors	= NULL;
-int pk_use_dh		= -1;
+int pk_use_enckey	= 0;
 
 static char *krb4_cc_name;
 
@@ -116,7 +120,7 @@ static struct getargs args[] = {
     { "renewable-life",	'r', arg_string, &renew_life,
       "renewable lifetime of tickets", "time" },
 
-    { "server", 	'S', arg_string, &server,
+    { "server", 	'S', arg_string, &server_str,
       "server to get ticket for", "principal" },
 
     { "start-time",	's', arg_string, &start_str,
@@ -146,6 +150,9 @@ static struct getargs args[] = {
     { "request-pac",	0,   arg_flag,	&pac_flag,
       "request a Windows PAC" },
 
+    { "password-file",	0,   arg_string, &password_file,
+      "read the password from a file" },
+
 #ifdef PKINIT
     {  "pk-user",	'C',	arg_string,	&pk_user_id,
        "principal's public/private/certificate identifier",
@@ -154,8 +161,9 @@ static struct getargs args[] = {
     {  "x509-anchors",	'D',  arg_string, &pk_x509_anchors,
        "directory with CA certificates", "directory" },
 
-    {  "pkinit-use-dh",       0,  arg_flag, &pk_use_dh,
-       "make pkinit use DH" },
+    {  "pk-use-enckey",	0,  arg_flag, &pk_use_enckey,
+       "Use RSA encrypted reply (instead of DH)" },
+
 #endif
     { "version", 	0,   arg_flag, &version_flag },
     { "help",		0,   arg_flag, &help_flag }
@@ -362,7 +370,7 @@ renew_validate(krb5_context context,
 	       krb5_deltat life)
 {
     krb5_error_code ret;
-    krb5_creds in, *out;
+    krb5_creds in, *out = NULL;
     krb5_kdc_flags flags;
 
     memset(&in, 0, sizeof(in));
@@ -377,18 +385,40 @@ renew_validate(krb5_context context,
 	krb5_warn(context, ret, "get_server");
 	goto out;
     }
+
+    if (renew) {
+	/* 
+	 * no need to check the error here, its only to be 
+	 * friendly to the user
+	 */
+	krb5_get_credentials(context, KRB5_GC_CACHED, cache, &in, &out);
+    }
+
     flags.i = 0;
     flags.b.renewable         = flags.b.renew = renew;
     flags.b.validate          = validate;
+
     if (forwardable_flag != -1)
 	flags.b.forwardable       = forwardable_flag;
+    else if (out)
+	flags.b.forwardable 	  = out->flags.b.forwardable;
+
     if (proxiable_flag != -1)
 	flags.b.proxiable         = proxiable_flag;
+    else if (out)
+	flags.b.proxiable 	  = out->flags.b.proxiable;
+
     if (anonymous_flag != -1)
 	flags.b.request_anonymous = anonymous_flag;
     if(life)
 	in.times.endtime = time(NULL) + life;
 
+    if (out) {
+	krb5_free_creds (context, out);
+	out = NULL;
+    }
+
+
     ret = krb5_get_kdc_cred(context,
 			    cache,
 			    flags,
@@ -435,12 +465,34 @@ get_new_tickets(krb5_context context,
 {
     krb5_error_code ret;
     krb5_get_init_creds_opt *opt;
-    krb5_addresses no_addrs;
     krb5_creds cred;
     char passwd[256];
     krb5_deltat start_time = 0;
     krb5_deltat renew = 0;
     char *renewstr = NULL;
+    krb5_enctype *enctype = NULL;
+
+    passwd[0] = '\0';
+
+    if (password_file) {
+	FILE *f;
+
+	if (strcasecmp("STDIN", password_file) == 0)
+	    f = stdin;
+	else
+	    f = fopen(password_file, "r");
+	if (f == NULL)
+	    krb5_errx(context, 1, "Failed to open the password file %s",
+		      password_file);
+
+	if (fgets(passwd, sizeof(passwd), f) == NULL)
+	    krb5_errx(context, 1, 
+		      "Failed to read password from file %s", password_file);
+	if (f != stdin)
+	    fclose(f);
+	passwd[strcspn(passwd, "\n")] = '\0';
+    }
+
 
     memset(&cred, 0, sizeof(cred));
 
@@ -461,27 +513,23 @@ get_new_tickets(krb5_context context,
 	krb5_get_init_creds_opt_set_pac_request(context, opt, 
 						pac_flag ? TRUE : FALSE);
     if (pk_user_id) {
-	int flags = 0;
-	if (pk_use_dh == 1)
-	    flags |= 1;
 	ret = krb5_get_init_creds_opt_set_pkinit(context, opt,
 						 principal,
 						 pk_user_id,
 						 pk_x509_anchors,
-						 flags,
 						 NULL,
 						 NULL,
-						 NULL);
+						 pk_use_enckey ? 2 : 0,
+						 krb5_prompter_posix,
+						 NULL,
+						 passwd);
 	if (ret)
 	    krb5_err(context, 1, ret, "krb5_get_init_creds_opt_set_pkinit");
     }
 
-    if (!addrs_flag) {
-	no_addrs.len = 0;
-	no_addrs.val = NULL;
-
-	krb5_get_init_creds_opt_set_address_list (opt, &no_addrs);
-    }
+    if (addrs_flag != -1)
+	krb5_get_init_creds_opt_set_addressless(context, opt, 
+						addrs_flag ? FALSE : TRUE);
 
     if (renew_life == NULL && renewable_flag)
 	renewstr = "1 month";
@@ -507,8 +555,8 @@ get_new_tickets(krb5_context context,
     }
 
     if(etype_str.num_strings) {
-	krb5_enctype *enctype = NULL;
 	int i;
+
 	enctype = malloc(etype_str.num_strings * sizeof(*enctype));
 	if(enctype == NULL)
 	    errx(1, "out of memory");
@@ -536,36 +584,39 @@ get_new_tickets(krb5_context context,
 					  principal,
 					  kt,
 					  start_time,
-					  server,
+					  server_str,
 					  opt);
 	krb5_kt_close(context, kt);
     } else if (pk_user_id) {
 	ret = krb5_get_init_creds_password (context,
 					    &cred,
 					    principal,
-					    NULL,
+					    passwd,
 					    krb5_prompter_posix,
 					    NULL,
 					    start_time,
-					    server,
+					    server_str,
 					    opt);
     } else if (!interactive) {
 	krb5_warnx(context, "Not interactive, failed to get initial ticket");
-	krb5_get_init_creds_opt_free(opt);
+	krb5_get_init_creds_opt_free(context, opt);
 	return 0;
     } else {
-	char *p, *prompt;
 
-	krb5_unparse_name (context, principal, &p);
-	asprintf (&prompt, "%s's Password: ", p);
-	free (p);
-
-	if (UI_UTIL_read_pw_string(passwd, sizeof(passwd)-1, prompt, 0)){
-	    memset(passwd, 0, sizeof(passwd));
-	    exit(1);
+	if (passwd[0] == '\0') {
+	    char *p, *prompt;
+	    
+	    krb5_unparse_name (context, principal, &p);
+	    asprintf (&prompt, "%s's Password: ", p);
+	    free (p);
+	    
+	    if (UI_UTIL_read_pw_string(passwd, sizeof(passwd)-1, prompt, 0)){
+		memset(passwd, 0, sizeof(passwd));
+		exit(1);
+	    }
+	    free (prompt);
 	}
 
-	free (prompt);
 	
 	ret = krb5_get_init_creds_password (context,
 					    &cred,
@@ -574,10 +625,10 @@ get_new_tickets(krb5_context context,
 					    krb5_prompter_posix,
 					    NULL,
 					    start_time,
-					    server,
+					    server_str,
 					    opt);
     }
-    krb5_get_init_creds_opt_free(opt);
+    krb5_get_init_creds_opt_free(context, opt);
 #ifdef KRB4
     if (ret == KRB5KRB_AP_ERR_V4_REPLY || ret == KRB5_KDC_UNREACH) {
 	int exit_val;
@@ -634,6 +685,9 @@ get_new_tickets(krb5_context context,
 
     krb5_free_cred_contents (context, &cred);
 
+    if (enctype)
+	free(enctype);
+
     return 0;
 }
 
@@ -687,21 +741,27 @@ renew_func(void *ptr)
     struct renew_ctx *ctx = ptr;
     krb5_error_code ret;
     time_t expire;
+    int new_tickets = 0;
 
+    if (renewable_flag) {
+	ret = renew_validate(ctx->context, renewable_flag, validate_flag,
+			     ctx->ccache, server_str, ctx->ticket_life);
+	if (ret)
+	    new_tickets = 1;
+    } else
+	new_tickets = 1;
 
-    ret = renew_validate(ctx->context, renewable_flag, validate_flag,
-			     ctx->ccache, server, ctx->ticket_life);
-    if (ret)
+    if (new_tickets)
 	get_new_tickets(ctx->context, ctx->principal, 
 			ctx->ccache, ctx->ticket_life, 0);
 
     if(get_v4_tgt || convert_524)
-	do_524init(ctx->context, ctx->ccache, NULL, server);
+	do_524init(ctx->context, ctx->ccache, NULL, server_str);
     if(do_afslog && k_hasafs())
 	krb5_afslog(ctx->context, ctx->ccache, NULL, NULL);
 
     expire = ticket_lifetime(ctx->context, ctx->ccache, ctx->principal,
-			     server) / 2;
+			     server_str) / 2;
     return expire + 1;
 }
 
@@ -712,7 +772,7 @@ main (int argc, char **argv)
     krb5_context context;
     krb5_ccache  ccache;
     krb5_principal principal;
-    int optind = 0;
+    int optidx = 0;
     krb5_deltat ticket_life = 0;
 
     setprogname (argv[0]);
@@ -723,7 +783,7 @@ main (int argc, char **argv)
     else if (ret)
 	errx(1, "krb5_init_context failed: %d", ret);
   
-    if(getarg(args, sizeof(args) / sizeof(args[0]), argc, argv, &optind))
+    if(getarg(args, sizeof(args) / sizeof(args[0]), argc, argv, &optidx))
 	usage(1);
     
     if (help_flag)
@@ -734,8 +794,8 @@ main (int argc, char **argv)
 	exit(0);
     }
 
-    argc -= optind;
-    argv += optind;
+    argc -= optidx;
+    argv += optidx;
 
     if (argv[0]) {
 	ret = krb5_parse_name (context, argv[0], &principal);
@@ -750,6 +810,20 @@ main (int argc, char **argv)
     if(fcache_version)
 	krb5_set_fcache_version(context, fcache_version);
 
+    if(renewable_flag == -1)
+	/* this seems somewhat pointless, but whatever */
+	krb5_appdefault_boolean(context, "kinit",
+				krb5_principal_get_realm(context, principal),
+				"renewable", FALSE, &renewable_flag);
+    if(get_v4_tgt == -1)
+	krb5_appdefault_boolean(context, "kinit", 
+				krb5_principal_get_realm(context, principal), 
+				"krb4_get_tickets", FALSE, &get_v4_tgt);
+    if(do_afslog == -1)
+	krb5_appdefault_boolean(context, "kinit", 
+				krb5_principal_get_realm(context, principal), 
+				"afslog", TRUE, &do_afslog);
+
     if(cred_cache) 
 	ret = krb5_cc_resolve(context, cred_cache, &ccache);
     else {
@@ -764,7 +838,7 @@ main (int argc, char **argv)
 	    setenv("KRB5CCNAME", s, 1);
 	    if (get_v4_tgt) {
 		int fd;
-		if (asprintf(&krb4_cc_name, "%s_XXXXXXXXXXX", TKT_ROOT) < 0)
+		if (asprintf(&krb4_cc_name, "%s_XXXXXX", TKT_ROOT) < 0)
 		    krb5_errx(context, 1, "out of memory");
 		if((fd = mkstemp(krb4_cc_name)) >= 0) {
 		    close(fd);
@@ -790,33 +864,8 @@ main (int argc, char **argv)
 
 	ticket_life = tmp;
     }
-    if(renewable_flag == -1)
-	/* this seems somewhat pointless, but whatever */
-	krb5_appdefault_boolean(context, "kinit",
-				krb5_principal_get_realm(context, principal),
-				"renewable", FALSE, &renewable_flag);
-    if(get_v4_tgt == -1)
-	krb5_appdefault_boolean(context, "kinit", 
-				krb5_principal_get_realm(context, principal), 
-				"krb4_get_tickets", FALSE, &get_v4_tgt);
-    if(do_afslog == -1)
-	krb5_appdefault_boolean(context, "kinit", 
-				krb5_principal_get_realm(context, principal), 
-				"afslog", TRUE, &do_afslog);
 
-    if (pk_x509_anchors == NULL)
-	krb5_appdefault_string(context, "kinit",
-			       krb5_principal_get_realm(context, principal), 
-			       "pkinit-anchors", NULL, &pk_x509_anchors);
-
-#ifdef PKINIT
-    if(pk_use_dh == -1)
-	krb5_appdefault_boolean(context, "kinit", 
-				krb5_principal_get_realm(context, principal), 
-				"pkinit-use-dh", FALSE, &pk_use_dh);
-#endif
-
-    if(!addrs_flag && extra_addresses.num_strings > 0)
+    if(addrs_flag == 0 && extra_addresses.num_strings > 0)
 	krb5_errx(context, 1, "specifying both extra addresses and "
 		  "no addresses makes no sense");
     {
@@ -836,7 +885,7 @@ main (int argc, char **argv)
 
     if(renew_flag || validate_flag) {
 	ret = renew_validate(context, renew_flag, validate_flag, 
-			     ccache, server, ticket_life);
+			     ccache, server_str, ticket_life);
 	exit(ret != 0);
     }
 
@@ -844,14 +893,14 @@ main (int argc, char **argv)
 	get_new_tickets(context, principal, ccache, ticket_life, 1);
 
     if(get_v4_tgt || convert_524)
-	do_524init(context, ccache, NULL, server);
+	do_524init(context, ccache, NULL, server_str);
     if(do_afslog && k_hasafs())
 	krb5_afslog(context, ccache, NULL, NULL);
     if(argc > 1) {
 	struct renew_ctx ctx;
 	time_t timeout;
 
-	timeout = ticket_lifetime(context, ccache, principal, server) / 2;
+	timeout = ticket_lifetime(context, ccache, principal, server_str) / 2;
 
 	ctx.context = context;
 	ctx.ccache = ccache;
