@@ -33,7 +33,7 @@
 
 #include "kadm5_locl.h"
 
-RCSID("$Id: create_s.c,v 1.22 2004/03/07 22:43:46 lha Exp $");
+RCSID("$Id: create_s.c 17445 2006-05-05 10:37:46Z lha $");
 
 static kadm5_ret_t
 get_default(kadm5_server_context *context, krb5_principal princ, 
@@ -56,14 +56,14 @@ get_default(kadm5_server_context *context, krb5_principal princ,
 static kadm5_ret_t
 create_principal(kadm5_server_context *context,
 		 kadm5_principal_ent_t princ,
-		 u_int32_t mask,
-		 hdb_entry *ent,
-		 u_int32_t required_mask,
-		 u_int32_t forbidden_mask)
+		 uint32_t mask,
+		 hdb_entry_ex *ent,
+		 uint32_t required_mask,
+		 uint32_t forbidden_mask)
 {
     kadm5_ret_t ret;
     kadm5_principal_ent_rec defrec, *defent;
-    u_int32_t def_mask;
+    uint32_t def_mask;
     
     if((mask & required_mask) != required_mask)
 	return KADM5_BAD_MASK;
@@ -74,7 +74,7 @@ create_principal(kadm5_server_context *context,
 	return KADM5_UNK_POLICY;
     memset(ent, 0, sizeof(*ent));
     ret  = krb5_copy_principal(context->context, princ->principal, 
-			       &ent->principal);
+			       &ent->entry.principal);
     if(ret)
 	return ret;
     
@@ -94,9 +94,9 @@ create_principal(kadm5_server_context *context,
     if(defent)
 	kadm5_free_principal_ent(context, defent);
     
-    ent->created_by.time = time(NULL);
+    ent->entry.created_by.time = time(NULL);
     ret = krb5_copy_principal(context->context, context->caller, 
-			      &ent->created_by.principal);
+			      &ent->entry.created_by.principal);
 
     return ret;
 }
@@ -104,10 +104,10 @@ create_principal(kadm5_server_context *context,
 kadm5_ret_t
 kadm5_s_create_principal_with_key(void *server_handle,
 				  kadm5_principal_ent_t princ,
-				  u_int32_t mask)
+				  uint32_t mask)
 {
     kadm5_ret_t ret;
-    hdb_entry ent;
+    hdb_entry_ex ent;
     kadm5_server_context *context = server_handle;
 
     ret = create_principal(context, princ, mask, &ent,
@@ -120,13 +120,14 @@ kadm5_s_create_principal_with_key(void *server_handle,
     if(ret)
 	goto out;
 
-    ent.kvno = 1;
+    if ((mask & KADM5_KVNO) == 0)
+	ent.entry.kvno = 1;
 
-    ret = hdb_seal_keys(context->context, context->db, &ent);
+    ret = hdb_seal_keys(context->context, context->db, &ent.entry);
     if (ret)
 	goto out;
     
-    kadm5_log_create (context, &ent);
+    kadm5_log_create (context, &ent.entry);
 
     ret = context->db->hdb_open(context->context, context->db, O_RDWR, 0);
     if(ret)
@@ -142,11 +143,11 @@ out:
 kadm5_ret_t
 kadm5_s_create_principal(void *server_handle,
 			 kadm5_principal_ent_t princ, 
-			 u_int32_t mask,
-			 char *password)
+			 uint32_t mask,
+			 const char *password)
 {
     kadm5_ret_t ret;
-    hdb_entry ent;
+    hdb_entry_ex ent;
     kadm5_server_context *context = server_handle;
 
     ret = create_principal(context, princ, mask, &ent,
@@ -159,18 +160,21 @@ kadm5_s_create_principal(void *server_handle,
     if(ret)
 	goto out;
 
-    ent.keys.len = 0;
-    ent.keys.val = NULL;
+    if ((mask & KADM5_KVNO) == 0)
+	ent.entry.kvno = 1;
 
-    ret = _kadm5_set_keys(context, &ent, password);
+    ent.entry.keys.len = 0;
+    ent.entry.keys.val = NULL;
+
+    ret = _kadm5_set_keys(context, &ent.entry, password);
     if (ret)
 	goto out;
 
-    ret = hdb_seal_keys(context->context, context->db, &ent);
+    ret = hdb_seal_keys(context->context, context->db, &ent.entry);
     if (ret)
 	goto out;
     
-    kadm5_log_create (context, &ent);
+    kadm5_log_create (context, &ent.entry);
 
     ret = context->db->hdb_open(context->context, context->db, O_RDWR, 0);
     if(ret)
