@@ -34,7 +34,7 @@
 #include "iprop.h"
 #include <rtbl.h>
 
-RCSID("$Id: ipropd_master.c,v 1.42 2005/05/23 17:38:46 lha Exp $");
+RCSID("$Id: ipropd_master.c 19233 2006-12-06 08:04:05Z lha $");
 
 static krb5_log_facility *log_facility;
 
@@ -106,7 +106,7 @@ struct slave {
     struct sockaddr_in addr;
     char *name;
     krb5_auth_context ac;
-    u_int32_t version;
+    uint32_t version;
     time_t seen;
     unsigned long flags;
 #define SLAVE_F_DEAD	0x1
@@ -127,8 +127,7 @@ check_acl (krb5_context context, const char *name)
     if (fp == NULL)
 	return 1;
     while (fgets(buf, sizeof(buf), fp) != NULL) {
-	if (buf[strlen(buf) - 1 ] == '\n')
-	    buf[strlen(buf) - 1 ] = '\0';
+	buf[strcspn(buf, "\r\n")] = '\0';
 	if (strcmp (buf, name) == 0) {
 	    ret = 0;
 	    break;
@@ -279,14 +278,14 @@ struct prop_context {
 };
 
 static int
-prop_one (krb5_context context, HDB *db, hdb_entry *entry, void *v)
+prop_one (krb5_context context, HDB *db, hdb_entry_ex *entry, void *v)
 {
     krb5_error_code ret;
     krb5_storage *sp;
     krb5_data data;
-    struct slave *slave = (struct slave *)v;
+    struct slave *s = (struct slave *)v;
 
-    ret = hdb_entry2value (context, entry, &data);
+    ret = hdb_entry2value (context, &entry->entry, &data);
     if (ret)
 	return ret;
     ret = krb5_data_realloc (&data, data.length + 4);
@@ -303,14 +302,14 @@ prop_one (krb5_context context, HDB *db, hdb_entry *entry, void *v)
     krb5_store_int32(sp, ONE_PRINC);
     krb5_storage_free(sp);
 
-    ret = krb5_write_priv_message (context, slave->ac, &slave->fd, &data);
+    ret = krb5_write_priv_message (context, s->ac, &s->fd, &data);
     krb5_data_free (&data);
     return ret;
 }
 
 static int
 send_complete (krb5_context context, slave *s,
-	       const char *database, u_int32_t current_version)
+	       const char *database, uint32_t current_version)
 {
     krb5_error_code ret;
     krb5_storage *sp;
@@ -413,13 +412,13 @@ send_are_you_there (krb5_context context, slave *s)
 
 static int
 send_diffs (krb5_context context, slave *s, int log_fd,
-	    const char *database, u_int32_t current_version)
+	    const char *database, uint32_t current_version)
 {
     krb5_storage *sp;
-    u_int32_t ver;
+    uint32_t ver;
     time_t timestamp;
     enum kadm_ops op;
-    u_int32_t len;
+    uint32_t len;
     off_t right, left;
     krb5_data data;
     int ret = 0;
@@ -433,8 +432,10 @@ send_diffs (krb5_context context, slave *s, int log_fd,
     sp = kadm5_log_goto_end (log_fd);
     right = krb5_storage_seek(sp, 0, SEEK_CUR);
     for (;;) {
-	if (kadm5_log_previous (sp, &ver, &timestamp, &op, &len))
-	    abort ();
+	ret = kadm5_log_previous (context, sp, &ver, &timestamp, &op, &len);
+	if (ret)
+	    krb5_err(context, 1, ret, 
+		     "send_diffs: failed to find previous entry");
 	left = krb5_storage_seek(sp, -16, SEEK_CUR);
 	if (ver == s->version)
 	    return 0;
@@ -476,7 +477,7 @@ send_diffs (krb5_context context, slave *s, int log_fd,
 
 static int
 process_msg (krb5_context context, slave *s, int log_fd,
-	     const char *database, u_int32_t current_version)
+	     const char *database, uint32_t current_version)
 {
     int ret = 0;
     krb5_data out;
@@ -533,7 +534,7 @@ process_msg (krb5_context context, slave *s, int log_fd,
 #define SLAVE_SEEN	"Last Seen"
 
 static void
-write_stats(krb5_context context, slave *slaves, u_int32_t current_version)
+write_stats(krb5_context context, slave *slaves, uint32_t current_version)
 {
     char str[100];
     rtbl_t tbl;
@@ -639,12 +640,12 @@ main(int argc, char **argv)
     int signal_fd, listen_fd;
     int log_fd;
     slave *slaves = NULL;
-    u_int32_t current_version = 0, old_version = 0;
+    uint32_t current_version = 0, old_version = 0;
     krb5_keytab keytab;
-    int optind;
+    int optidx;
     char **files;
     
-    optind = krb5_program_setup(&context, argc, argv, args, num_args, NULL);
+    optidx = krb5_program_setup(&context, argc, argv, args, num_args, NULL);
     
     if(help_flag)
 	krb5_std_usage(0, args, num_args);
@@ -717,7 +718,7 @@ main(int argc, char **argv)
 	fd_set readset;
 	int max_fd = 0;
 	struct timeval to = {30, 0};
-	u_int32_t vers;
+	uint32_t vers;
 
 	if (signal_fd >= FD_SETSIZE || listen_fd >= FD_SETSIZE)
 	    krb5_errx (context, 1, "fd too large");
