@@ -33,7 +33,7 @@
 
 #include "iprop.h"
 
-RCSID("$Id: ipropd_slave.c,v 1.39 2005/05/23 17:39:35 lha Exp $");
+RCSID("$Id: ipropd_slave.c 19078 2006-11-20 18:12:41Z lha $");
 
 static krb5_log_facility *log_facility;
 static char *server_time_lost = "5 min";
@@ -115,7 +115,7 @@ get_creds(krb5_context context, const char *keytab_str,
     ret = krb5_get_init_creds_keytab(context, &creds, client, keytab,
 				     0, server, init_opts);
     free (server);
-    krb5_get_init_creds_opt_free(init_opts);
+    krb5_get_init_creds_opt_free(context, init_opts);
     if(ret) krb5_err(context, 1, ret, "krb5_get_init_creds");
     
     ret = krb5_kt_close(context, keytab);
@@ -133,7 +133,7 @@ get_creds(krb5_context context, const char *keytab_str,
 
 static void
 ihave (krb5_context context, krb5_auth_context auth_context,
-       int fd, u_int32_t version)
+       int fd, uint32_t version)
 {
     int ret;
     u_char buf[8];
@@ -302,14 +302,16 @@ receive_everything (krb5_context context, int fd,
 	krb5_ret_int32 (sp, &opcode);
 	if (opcode == ONE_PRINC) {
 	    krb5_data fake_data;
-	    hdb_entry entry;
+	    hdb_entry_ex entry;
 
 	    krb5_storage_free(sp);
 
 	    fake_data.data   = (char *)data.data + 4;
 	    fake_data.length = data.length - 4;
 
-	    ret = hdb_value2entry (context, &fake_data, &entry);
+	    memset(&entry, 0, sizeof(entry));
+
+	    ret = hdb_value2entry (context, &fake_data, &entry.entry);
 	    if (ret)
 		krb5_err (context, 1, ret, "hdb_value2entry");
 	    ret = mydb->hdb_store(server_context->context,
@@ -384,13 +386,6 @@ static struct getargs args[] = {
 
 static int num_args = sizeof(args) / sizeof(args[0]);
 
-static void
-usage (int code, struct getargs *args, int num_args)
-{
-    arg_printusage (args, num_args, NULL, "master");
-    exit (code);
-}
-
 int
 main(int argc, char **argv)
 {
@@ -405,13 +400,12 @@ main(int argc, char **argv)
     krb5_principal server;
     char **files;
 
-    int optind;
     const char *master;
     
-    optind = krb5_program_setup(&context, argc, argv, args, num_args, usage);
+    krb5_program_setup(&context, argc, argv, args, num_args, NULL);
     
     if(help_flag)
-	usage (0, args, num_args);
+	krb5_std_usage(0, args, num_args);
     if(version_flag) {
 	print_version(NULL);
 	exit(0);
@@ -433,7 +427,7 @@ main(int argc, char **argv)
     argv += optind;
 
     if (argc != 1)
-	usage (1, args, num_args);
+	krb5_std_usage(1, args, num_args);
 
     master = argv[0];
 
@@ -492,7 +486,6 @@ main(int argc, char **argv)
 	   server_context->log_context.version);
 
     for (;;) {
-	int ret;
 	krb5_data out;
 	krb5_storage *sp;
 	int32_t tmp;
