@@ -33,7 +33,7 @@
 
 #include "krb5_locl.h"
 
-RCSID("$Id: read_message.c,v 1.9 2004/05/25 21:40:33 lha Exp $");
+RCSID("$Id: read_message.c 17442 2006-05-05 09:31:15Z lha $");
 
 krb5_error_code KRB5_LIB_FUNCTION
 krb5_read_message (krb5_context context,
@@ -41,8 +41,10 @@ krb5_read_message (krb5_context context,
 		   krb5_data *data)
 {
     krb5_error_code ret;
-    u_int32_t len;
-    u_int8_t buf[4];
+    uint32_t len;
+    uint8_t buf[4];
+
+    krb5_data_zero(data);
 
     ret = krb5_net_read (context, p_fd, buf, 4);
     if(ret == -1) {
@@ -50,10 +52,8 @@ krb5_read_message (krb5_context context,
 	krb5_clear_error_string (context);
 	return ret;
     }
-    if(ret < 4) {
-	data->length = 0;
+    if(ret < 4)
 	return HEIM_ERR_EOF;
-    }
     len = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
     ret = krb5_data_alloc (data, len);
     if (ret)
