@@ -35,7 +35,7 @@
 #endif
 #include "roken.h"
 
-RCSID("$Id: strftime.c,v 1.14 2005/04/12 11:29:08 lha Exp $");
+RCSID("$Id: strftime.c 15413 2005-06-16 16:55:55Z lha $");
 
 static const char *abb_weekdays[] = {
     "Sun",
@@ -290,7 +290,7 @@ strftime (char *buf, size_t maxsize, const char *format,
 		    
 	    case 's' :
 		ret = snprintf (buf, maxsize - n,
-				"%d", (int)mktime((struct tm *)tm));
+				"%d", (int)mktime(rk_UNCONST(tm)));
 		break;
 	    case 'S' :
 		ret = snprintf (buf, maxsize - n,
